@@ -14,5 +14,6 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-prefix = '/home/jani/hack/olpc/sugar/new/sugar-jhbuild/build'
-data_path = '/home/jani/hack/olpc/sugar/new/sugar-jhbuild/build/share/sugar/data'
+prefix = '/usr/local'
+data_path = '/usr/local/share/sugar/data'
+shell_path = '/usr/local/share/sugar/shell'
\ No newline at end of file
