@@ -24,7 +24,7 @@ STATE_ACTIVATING = 0
 STATE_ACTIVATED  = 1
 STATE_INACTIVE   = 2
 
-_nm_state_to_state = {
+nm_state_to_state = {
     nmclient.DEVICE_STATE_ACTIVATING : STATE_ACTIVATING,
     nmclient.DEVICE_STATE_ACTIVATED  : STATE_ACTIVATED,
     nmclient.DEVICE_STATE_INACTIVE   : STATE_INACTIVE
