@@ -14,7 +14,6 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
-import random
 from gettext import gettext as _
 import logging
 
@@ -22,36 +21,40 @@ import hippo
 import gobject
 import gtk
 
-from sugar.graphics.icon import CanvasIcon
+from sugar.graphics.icon import CanvasIcon, Icon
+from sugar.graphics.xocolor import XoColor
 from sugar.graphics import style
 from sugar.graphics.icon import get_icon_state
-from sugar.graphics import style
 from sugar.graphics import palette
 from sugar.graphics import iconentry
 from sugar.graphics.menuitem import MenuItem
 from sugar import profile
 
 from model import accesspointmodel
-from model.devices.network import mesh
 from model.devices.network import wireless
+from model import shellmodel
 from hardware import hardwaremanager
 from hardware import nmclient
 from view.BuddyIcon import BuddyIcon
-from view.pulsingicon import PulsingIcon
+from view.pulsingicon import CanvasPulsingIcon
 from view.home.snowflakelayout import SnowflakeLayout
 from view.home.spreadlayout import SpreadLayout
+import view.Shell
 
-from hardware.nmclient import NM_802_11_CAP_PROTO_WEP, NM_802_11_CAP_PROTO_WPA, NM_802_11_CAP_PROTO_WPA2
+from hardware.nmclient import NM_802_11_CAP_PROTO_WEP, \
+    NM_802_11_CAP_PROTO_WPA, NM_802_11_CAP_PROTO_WPA2
 
 
 _ICON_NAME = 'network-wireless'
 
-class AccessPointView(PulsingIcon):
+class AccessPointView(CanvasPulsingIcon):
     def __init__(self, model, mesh_device=None):
-        PulsingIcon.__init__(self, size=style.STANDARD_ICON_SIZE, cache=True)
+        CanvasPulsingIcon.__init__(self, size=style.STANDARD_ICON_SIZE,
+                                   cache=True)
         self._model = model
         self._meshdev = mesh_device
         self._disconnect_item = None
+        self._connect_item = None
         self._greyed_out = False
 
         self.connect('activated', self._activate_cb)
@@ -60,9 +63,18 @@ class AccessPointView(PulsingIcon):
         model.connect('notify::name', self._name_changed_cb)
         model.connect('notify::state', self._state_changed_cb)
 
-        (stroke, fill) = model.get_nm_network().get_colors()
-        self._device_stroke = stroke
-        self._device_fill = fill
+        pulse_color = XoColor('%s,%s' % (style.COLOR_BUTTON_GREY.get_svg(),
+                                         style.COLOR_TRANSPARENT.get_svg()))
+        self.props.pulse_color = pulse_color
+
+        # Update badge
+        caps = model.props.capabilities
+        if model.get_nm_network().is_favorite():
+            self.props.badge_name = "emblem-favorite"
+        elif (caps & NM_802_11_CAP_PROTO_WEP) or \
+                (caps & NM_802_11_CAP_PROTO_WPA) or \
+                (caps & NM_802_11_CAP_PROTO_WPA2):
+            self.props.badge_name = "emblem-locked"
 
         self._palette = self._create_palette()
         self.set_palette(self._palette)
@@ -71,27 +83,31 @@ class AccessPointView(PulsingIcon):
         self._update_name()
         self._update_state()
 
-        # Update badge
-        caps = model.props.capabilities
-        if model.get_nm_network().is_favorite():
-            self.props.badge_name = "emblem-favorite"
-        elif (caps & NM_802_11_CAP_PROTO_WEP) or (caps & NM_802_11_CAP_PROTO_WPA) or (caps & NM_802_11_CAP_PROTO_WPA2):
-            self.props.badge_name = "emblem-locked"
-
     def _create_palette(self):
-        p = palette.Palette(self._model.props.name, menu_after_content=True)
-        if not self._meshdev:
-            return p
+        icon_name = get_icon_state(_ICON_NAME, self._model.props.strength)
+        palette_icon = Icon(icon_name=icon_name,
+                            icon_size=style.STANDARD_ICON_SIZE,
+                            badge_name=self.props.badge_name)
+        ap_color = self._model.get_nm_network().get_colors()
+        palette_icon.props.xo_color = XoColor('%s,%s' % ap_color)
+                                              
+        p = palette.Palette(primary_text=self._model.props.name,
+                            icon=palette_icon)
+
+        self._connect_item = MenuItem(_('Connect'), 'dialog-ok')
+        self._connect_item.connect('activate', self._activate_cb)
+        p.menu.append(self._connect_item)
 
         # Only show disconnect when there's a mesh device, because mesh takes
-        # priority over the normal wireless device.  NM doesn't have a "disconnect"
-        # method for a device either (for various reasons) so this doesn't
-        # have a good mapping 
-        self._disconnect_item = gtk.MenuItem(_('Disconnect...'))
-        self._disconnect_item.connect('activate', self._disconnect_activate_cb)
-        p.menu.append(self._disconnect_item)
-        if self._model.props.state == accesspointmodel.STATE_CONNECTED:
-            self._disconnect_item.show()
+        # priority over the normal wireless device. NM doesn't have a
+        # "disconnect" method for a device either (for various reasons)
+        # so this doesn't have a good mapping
+        if self._meshdev:
+            self._disconnect_item = MenuItem(_('Disconnect'), 'media-eject')
+            self._disconnect_item.connect('activate',
+                                          self._disconnect_activate_cb)
+            p.menu.append(self._disconnect_item)
+
         return p
 
     def _disconnect_activate_cb(self, menuitem):
@@ -99,6 +115,8 @@ class AccessPointView(PulsingIcon):
         network_manager = hardwaremanager.get_network_manager()
         if network_manager and self._meshdev:
             network_manager.set_active_device(self._meshdev)
+            self._palette.props.secondary_text = _('Disconnecting...')
+            self.props.pulsing = False
 
     def _strength_changed_cb(self, model, pspec):
         self._update_icon()
@@ -117,46 +135,43 @@ class AccessPointView(PulsingIcon):
             network_manager.set_active_device(device, network)
 
     def _update_name(self):
-        self._palette.set_primary_text(self._model.props.name)
+        self._palette.props.primary_text = self._model.props.name
 
     def _update_icon(self):
         icon_name = get_icon_state(_ICON_NAME, self._model.props.strength)
         if icon_name:
-            self.props.icon_name = icon_name        
+            self.props.icon_name = icon_name
+            icon = self._palette.props.icon
+            icon.props.icon_name = icon_name
 
     def _update_state(self):
         if self._model.props.state == accesspointmodel.STATE_CONNECTING:
             if self._disconnect_item:
-                self._disconnect_item.hide()
-            self.props.pulse_time = 1.0
-            self.props.colors = [
-                [ style.Color(self._device_stroke).get_svg(),
-                  style.Color(self._device_fill).get_svg() ],
-                [ style.Color(self._device_stroke).get_svg(),
-                  '#e2e2e2' ]
-            ]
+                self._disconnect_item.show()
+            self._connect_item.hide()
+            self._palette.props.secondary_text = _('Connecting...')
+            self.props.pulsing = True
         elif self._model.props.state == accesspointmodel.STATE_CONNECTED:
             if self._disconnect_item:
                 self._disconnect_item.show()
-            self.props.pulse_time = 0.0
-            self.props.colors = [
-                [ '#ffffff',
-                  style.Color(self._device_fill).get_svg() ],
-                [ '#ffffff',
-                  style.Color(self._device_fill).get_svg() ]
-            ]
+            self._connect_item.hide()
+            # TODO: show the channel number
+            self._palette.props.secondary_text = _('Connected')
+            self.props.pulsing = False
         elif self._model.props.state == accesspointmodel.STATE_NOTCONNECTED:
             if self._disconnect_item:
                 self._disconnect_item.hide()
-            self.props.pulse_time = 0.0
-            self.props.colors = [
-                [ style.Color(self._device_stroke).get_svg(),
-                    style.Color(self._device_fill).get_svg() ]
-            ]
+            self._connect_item.show()
+            # TODO: show the channel number
+            self._palette.props.secondary_text = None
+            self.props.pulsing = False
 
         if self._greyed_out:
-            self.props.pulse_time = 0.0
-            self.props.colors = [['#D5D5D5', '#D5D5D5']]
+            self.props.pulsing = False
+            self.props.base_color = XoColor('#D5D5D5,#D5D5D5')
+        else:
+            self.props.base_color = XoColor('%s,%s' % \
+                    self._model.get_nm_network().get_colors())
 
     def set_filter(self, query):
         self._greyed_out = self._model.props.name.lower().find(query) == -1
@@ -164,12 +179,12 @@ class AccessPointView(PulsingIcon):
 
 _MESH_ICON_NAME = 'network-mesh'
 
-class MeshDeviceView(PulsingIcon):
+class MeshDeviceView(CanvasPulsingIcon):
     def __init__(self, nm_device, channel):
         if not channel in [1, 6, 11]:
             raise ValueError("Invalid channel %d" % channel)
 
-        PulsingIcon.__init__(self, size=style.STANDARD_ICON_SIZE,
+        CanvasPulsingIcon.__init__(self, size=style.STANDARD_ICON_SIZE,
                              icon_name=_MESH_ICON_NAME, cache=True)
 
         self._nm_device = nm_device
@@ -181,18 +196,20 @@ class MeshDeviceView(PulsingIcon):
         self._palette = self._create_palette()
         self.set_palette(self._palette)
 
-        mycolor = profile.get_color()
-        self._device_fill = mycolor.get_fill_color()
-        self._device_stroke = mycolor.get_stroke_color()
+        pulse_color = XoColor('%s,%s' % (style.COLOR_BUTTON_GREY.get_svg(),
+                                         style.COLOR_TRANSPARENT.get_svg()))
+        self.props.pulse_color = pulse_color
 
         self.connect('activated', self._activate_cb)
 
         self._nm_device.connect('state-changed', self._state_changed_cb)
-        self._nm_device.connect('activation-stage-changed', self._state_changed_cb)
+        self._nm_device.connect('activation-stage-changed',
+                                self._state_changed_cb)
         self._update_state()
 
     def _create_palette(self):
-        p = palette.Palette(_("Mesh Network") + " " + str(self.channel), menu_after_content=True)
+        p = palette.Palette(_("Mesh Network") + " " + str(self.channel),
+                            menu_after_content=True)
 
         self._disconnect_item = gtk.MenuItem(_('Disconnect...'))
         self._disconnect_item.connect('activate', self._disconnect_activate_cb)
@@ -221,47 +238,33 @@ class MeshDeviceView(PulsingIcon):
     def _update_state(self):
         state = self._nm_device.get_state()
         chan = wireless.freq_to_channel(self._nm_device.get_frequency())
-        if self._greyed_out:
-            self.props.colors = [['#D5D5D5', '#D5D5D5']]
-        elif state == nmclient.DEVICE_STATE_ACTIVATING and chan == self.channel:
+        if state == nmclient.DEVICE_STATE_ACTIVATING and chan == self.channel:
             self._disconnect_item.hide()
-            self.props.pulse_time = 0.75
-            self.props.colors = [
-                [ style.Color(self._device_stroke).get_svg(),
-                    style.Color(self._device_fill).get_svg() ],
-                [ style.Color(self._device_stroke).get_svg(),
-                    '#e2e2e2' ]
-            ]
+            self.props.pulsing = True
         elif state == nmclient.DEVICE_STATE_ACTIVATED and chan == self.channel:
             self._disconnect_item.show()
-            self.props.pulse_time = 0.0
-            self.props.colors = [
-                [ '#ffffff',
-                    style.Color(self._device_fill).get_svg() ],
-                [ '#ffffff',
-                    style.Color(self._device_fill).get_svg() ]
-            ]
+            self.props.pulsing = False
         elif state == nmclient.DEVICE_STATE_INACTIVE or chan != self.channel:
             self._disconnect_item.hide()
-            self.props.pulse_time = 0.0
-            self.props.colors = [
-                [ style.Color(self._device_stroke).get_svg(),
-                  style.Color(self._device_fill).get_svg() ]
-            ]
+            self.props.pulsing = False
+
+        if self._greyed_out:
+            self.props.pulsing = False
+            self.props.base_color = XoColor('#D5D5D5,#D5D5D5')
         else:
-            raise RuntimeError("Shouldn't get here")
+            self.props.base_color = profile.get_color()
 
     def set_filter(self, query):
         self._greyed_out = (query != '')
         self._update_state()
 
 class ActivityView(hippo.CanvasBox):
-    def __init__(self, shell, model):
+    def __init__(self, model):
         hippo.CanvasBox.__init__(self)
 
-        self._shell = shell
         self._model = model
         self._icons = {}
+        self._palette = None
 
         self._layout = SnowflakeLayout()
         self.set_layout(self._layout)
@@ -286,7 +289,11 @@ class ActivityView(hippo.CanvasBox):
         return icon
 
     def _create_palette(self):
-        p = palette.Palette(self._model.activity.props.name)
+        p_icon = Icon(file=self._model.get_icon_name(),
+                      xo_color=self._model.get_color())
+        p_icon.props.icon_size = gtk.ICON_SIZE_LARGE_TOOLBAR
+        p = palette.Palette(None, primary_text=self._model.activity.props.name,
+                            icon=p_icon)
 
         private = self._model.activity.props.private
         joined = self._model.activity.props.joined
@@ -322,18 +329,18 @@ class ActivityView(hippo.CanvasBox):
 
     def _clicked_cb(self, item):
         bundle_id = self._model.get_bundle_id()
-        self._shell.join_activity(bundle_id, self._model.get_id())
+        view.Shell.get_instance().join_activity(bundle_id, self._model.get_id())
 
     def set_filter(self, query):
         text_to_check = self._model.activity.props.name.lower() + \
                 self._model.activity.props.type.lower()
         if text_to_check.find(query) == -1:
             self._icon.props.stroke_color = '#D5D5D5'
-            self._icon.props.fill_color = '#E5E5E5'
+            self._icon.props.fill_color = style.COLOR_TRANSPARENT.get_svg()
         else:
             self._icon.props.xo_color = self._model.get_color()
 
-        for key, icon in self._icons.iteritems():
+        for icon in self._icons.itervalues():
             if hasattr(icon, 'set_filter'):
                 icon.set_filter(query)
 
@@ -372,24 +379,29 @@ class MeshToolbar(gtk.Toolbar):
         self._add_separator()
 
         tool_item = gtk.ToolItem()
-        tool_item.set_expand(True)
         self.insert(tool_item, -1)
         tool_item.show()
 
-        self._search_entry = iconentry.IconEntry()
-        self._search_entry.set_icon_from_name(iconentry.ICON_ENTRY_PRIMARY, 'system-search')
-        self._search_entry.add_clear_button()
-        self._search_entry.connect('activate', self._entry_activated_cb)
-        self._search_entry.connect('changed', self._entry_changed_cb)
-        tool_item.add(self._search_entry)
-        self._search_entry.show()
+        self.search_entry = iconentry.IconEntry()
+        self.search_entry.set_icon_from_name(iconentry.ICON_ENTRY_PRIMARY,
+                                             'system-search')
+        self.search_entry.add_clear_button()
+        self.search_entry.set_width_chars(25)
+        self.search_entry.connect('activate', self._entry_activated_cb)
+        self.search_entry.connect('changed', self._entry_changed_cb)
+        tool_item.add(self.search_entry)
+        self.search_entry.show()
 
-        self._add_separator()
+        self._add_separator(expand=True)
 
-    def _add_separator(self):
+    def _add_separator(self, expand=False):
         separator = gtk.SeparatorToolItem()
-        separator.set_size_request(style.GRID_CELL_SIZE, style.GRID_CELL_SIZE)
         separator.props.draw = False
+        if expand:
+            separator.set_expand(True)
+        else:
+            separator.set_size_request(style.GRID_CELL_SIZE,
+                                       style.GRID_CELL_SIZE)
         self.insert(separator, -1)
         separator.show()
 
@@ -414,15 +426,15 @@ class MeshToolbar(gtk.Toolbar):
     def _autosearch_timer_cb(self):
         logging.debug('_autosearch_timer_cb')
         self._autosearch_timer = None
-        self._search_entry.activate()
+        self.search_entry.activate()
         return False
 
-class MeshBox(hippo.CanvasBox):
-    def __init__(self, shell):
-        hippo.CanvasBox.__init__(self)
+class MeshBox(gtk.VBox):
+    __gtype_name__ = 'SugarMeshBox'
+    def __init__(self):
+        gobject.GObject.__init__(self)
 
-        self._shell = shell
-        self._model = shell.get_model().get_mesh()
+        self._model = shellmodel.get_instance().get_mesh()
         self._buddies = {}
         self._activities = {}
         self._access_points = {}
@@ -430,13 +442,20 @@ class MeshBox(hippo.CanvasBox):
         self._buddy_to_activity = {}
         self._suspended = True
         self._query = ''
-
+        self._owner_icon = None
+            
         self._toolbar = MeshToolbar()
         self._toolbar.connect('query-changed', self._toolbar_query_changed_cb)
-        self.append(hippo.CanvasWidget(widget=self._toolbar))
+        self.pack_start(self._toolbar, expand=False)
+        self._toolbar.show()
 
-        self._layout_box = hippo.CanvasBox(background_color=0xe2e2e2ff)
-        self.append(self._layout_box, hippo.PACK_EXPAND)
+        canvas = hippo.Canvas()
+        self.add(canvas)
+        canvas.show()
+
+        self._layout_box = hippo.CanvasBox( \
+                background_color=style.COLOR_WHITE.get_int())
+        canvas.set_root(self._layout_box)
 
         self._layout = SpreadLayout()
         self._layout_box.set_layout(self._layout)
@@ -463,23 +482,33 @@ class MeshBox(hippo.CanvasBox):
                             self._access_point_removed_cb)
 
         if self._model.get_mesh():
-            self._mesh_added_cb(self._model, self._model.get_mesh())
+            self.__mesh_added_cb(self._model, self._model.get_mesh())
 
-        self._model.connect('mesh-added',
-                            self._mesh_added_cb)
-        self._model.connect('mesh-removed',
-                            self._mesh_removed_cb)
+        self._model.connect('mesh-added', self.__mesh_added_cb)
+        self._model.connect('mesh-removed', self.__mesh_removed_cb)
 
-    def _mesh_added_cb(self, model, meshdev):
+    def __mesh_added_cb(self, model, meshdev):
         self._add_mesh_icon(meshdev, 1)
         self._add_mesh_icon(meshdev, 6)
         self._add_mesh_icon(meshdev, 11)
 
-    def _mesh_removed_cb(self, model):
+    def __mesh_removed_cb(self, model):
         self._remove_mesh_icon(1)
         self._remove_mesh_icon(6)
         self._remove_mesh_icon(11)
 
+    def do_size_allocate(self, allocation):
+        width = allocation.width        
+        height = allocation.height
+
+        min_w_, icon_width = self._owner_icon.get_width_request()
+        min_h_, icon_height = self._owner_icon.get_height_request(icon_width)
+        x = (width - icon_width) / 2
+        y = (height - icon_height) / 2 - style.GRID_CELL_SIZE
+        self._layout.move(self._owner_icon, x, y)
+
+        gtk.VBox.do_size_allocate(self, allocation)
+
     def _buddy_added_cb(self, model, buddy_model):
         self._add_alone_buddy(buddy_model)
 
@@ -519,12 +548,10 @@ class MeshBox(hippo.CanvasBox):
         del self._mesh[channel]
 
     def _add_alone_buddy(self, buddy_model):
-        icon = BuddyIcon(self._shell, buddy_model)
+        icon = BuddyIcon(buddy_model)
         if buddy_model.is_owner():
-            vertical_offset = - style.GRID_CELL_SIZE
-            self._layout.add_center(icon, vertical_offset)
-        else:
-            self._layout.add(icon)
+            self._owner_icon = icon
+        self._layout.add(icon)
 
         if hasattr(icon, 'set_filter'):
             icon.set_filter(self._query)
@@ -547,8 +574,6 @@ class MeshBox(hippo.CanvasBox):
                     activity.remove_buddy_icon(key)
 
     def _move_buddy(self, buddy_model, activity_model):
-        key = buddy_model.get_key()
-
         self._remove_buddy(buddy_model)
 
         if activity_model == None:
@@ -556,15 +581,14 @@ class MeshBox(hippo.CanvasBox):
         elif activity_model.get_id() in self._activities:
             activity = self._activities[activity_model.get_id()]
 
-            icon = BuddyIcon(self._shell, buddy_model,
-                             style.SMALL_ICON_SIZE)
+            icon = BuddyIcon(buddy_model, style.STANDARD_ICON_SIZE)
             activity.add_buddy_icon(buddy_model.get_key(), icon)
 
             if hasattr(icon, 'set_filter'):
                 icon.set_filter(self._query)
 
     def _add_activity(self, activity_model):
-        icon = ActivityView(self._shell, activity_model)
+        icon = ActivityView(activity_model)
         self._layout.add(icon)
 
         if hasattr(icon, 'set_filter'):
@@ -612,4 +636,4 @@ class MeshBox(hippo.CanvasBox):
                 icon.set_filter(self._query)
 
     def focus_search_entry(self):
-        self._toolbar._search_entry.grab_focus()
+        self._toolbar.search_entry.grab_focus()
